// chuk_com.c/
// functions to initialize and read wired nunchuk
// tincantools.co Trainer and Nintendo Wii or clone nunchuk
// tests performed with Nintendo Wii and Nyko Kama nunchuks
// 29.10.2010 09:09:42
// djl
// rev 0.2

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <linux/i2c-dev.h>

char rec_bytes;
unsigned char joy_x;
unsigned char joy_y;
unsigned char z_button;
unsigned char c_button;

unsigned long accel_x;
unsigned long accel_y;
unsigned long accel_z;

char filename[20];

// Nunchuk init codes
char write_buf_1[2] = {0xF0, 0x55};
char write_buf_2[3] = {0xFB, 0x00};
char write_buf_3[1] = {0x00};

char buf[7];
char data_buf[7];

int file;
int adapter_nr = 2; /* Beagleboard I2C2 Bus */
int i2c_addr = 0x52; /* nunchuk I2C address */
int i;
int read_bytes = 6;

/* I2C busses, u-boot are 0 - 2, userspace is 1 - 3 */

/* i2c_bus_init() open i2c bus 2 if available */
void i2c_bus_init()
{
	sprintf(filename, "/dev/i2c-2");
   if ((file = open(filename,O_RDWR)) < 0) {
        printf("Failed to open the bus.");
        /* ERROR HANDLING; you can check errno to see what went wrong */
		if (file >= 0)
		close(file);
        exit(1);
      } else {
        printf("Opened /dev/i2c-%d\n",adapter_nr);
		}
 }
 
 /* Wake up nunchuk */
void i2c_chuk_init()
{
	i2c_addr = 0x52;
    if (ioctl(file,I2C_SLAVE, i2c_addr) < 0)
    {
        printf("Failed to acquire bus access and/or talk to slave.\n");
        /* ERROR HANDLING; you can check errno to see what went wrong */
		if (file >= 0)
		close(file);
        exit(1);
    } else {
		printf("Comm to chuk is open.\n");
		}
    if (write(file, write_buf_1, 2) != 2) /* first init string */
    {
		printf("I2C Send %x Failed\n", i2c_addr);
		close(file);
        exit(1);

	}else {
		printf("I2C send buf[1] success.\n");
		}
	if (write(file, write_buf_2, 2) != 2) /* second init string */
    {
		printf("I2C Send %x Failed\n", i2c_addr);
		close(file);
        exit(1);
	}else {
		printf("I2C send buf[2] success.\n");
		}
 }
 
 /*i2c_chuk_read */
void i2c_chuk_read()
{
	read_bytes = 6;
	if (write(file, write_buf_3, 1) != 1) /* send to format data */
    {
		printf("I2C Send %x Failed\n", i2c_addr);
		close(file);
        exit(1);
	}
	// delay for adjusting reporting speed. Bad data without.
	usleep(10000); // 100k uS = 0.1S, 1k uSec = 0.001S
	if (read(file,buf,read_bytes) != read_bytes)
	{
        printf("I2C Send %x Failed\n",i2c_addr);
    }
}

/* extract data */
/* joy_x, joy_y, accel_x, accel_y, accel_z,
 * bit[0], button Z, bit[1] Button C  pressed = low */
void chuk_data()
{

	// buf[5] contains button and 2 bits each axis of accel data
//	const int buf[] = {0x00};
	z_button = 0;
	c_button = 0;
	accel_x  = 0;
	accel_y  = 0;
	accel_z  = 0;
	i2c_chuk_read();	// call i2c_chuk_read	
	// Joysticks data
	joy_x = buf[0];
	joy_y = buf[1];
	// acceleronometer data
	accel_x += ((buf[2]) << 2);
	accel_y += ((buf[3]) << 2);
	accel_z += ((buf[4]) << 2);
	
	// Buttons Z and C data
	if ((buf[5] & 0x01) !=0) {
		z_button = 1;
		data_buf[5] = z_button;
	}
	if ((buf[5] & 0x02) !=0) {
		c_button = 1;
		data_buf[6] = c_button;
	accel_x += ((buf[5]) >> 2) & 0x03;
	accel_y += ((buf[5]) >> 4) & 0x03;
	accel_z += ((buf[5]) >> 6) & 0x03;
	}
	// optional locak print for trouble shooting.
 /*       printf("Joy_x: %d, ", joy_x);	
	    printf("Joy_y: %d, ", joy_y);	
        printf("Accel_x: %ld, ", accel_x);	
        printf("Accel_y: %ld, ", accel_y);	
        printf("Accel_z: %ld, ", accel_z);	
        printf("B_Z: %d, ", z_button);	
        printf("B_C: %d, \n", c_button);	
*/
}
